@@ -36,22 +36,22 @@ tests = []
 
 class TestsZeros(unittest.TestCase):
 
-    def test_1(self):
+    def test_basic(self):
         for default_endian in 'big', 'little':
             _set_default_endian(default_endian)
             a = zeros(0)
             self.assertEqual(a, bitarray())
             self.assertEqual(a.endian(), default_endian)
 
-            b  = zeros(0, endian=None)
-            self.assertEqual(b.endian(), default_endian)
+            a = zeros(0, endian=None)
+            self.assertEqual(len(a), 0)
+            self.assertEqual(a.endian(), default_endian)
 
             for n in range(100):
                 a = zeros(n)
                 self.assertEqual(len(a), n)
                 self.assertFalse(a.any())
                 self.assertEqual(a.count(1), 0)
-                self.assertEqual(a, n * bitarray('0'))
                 self.assertEqual(a, bitarray(n * '0'))
 
             for endian in 'big', 'little':
@@ -77,28 +77,35 @@ tests.append(TestsZeros)
 
 # ---------------------------------------------------------------------------
 
-class TestsRandom(unittest.TestCase):
+class TestsURandom(unittest.TestCase):
 
-    def test_1(self):
+    def test_basic(self):
         for default_endian in 'big', 'little':
             _set_default_endian(default_endian)
             a = urandom(0)
             self.assertEqual(a, bitarray())
             self.assertEqual(a.endian(), default_endian)
 
-            b  = urandom(0, endian=None)
-            self.assertEqual(b.endian(), default_endian)
+            a = urandom(7, endian=None)
+            self.assertEqual(len(a), 7)
+            self.assertEqual(a.endian(), default_endian)
 
-            for n in range(100):
+            for n in range(50):
                 a = urandom(n)
                 self.assertEqual(len(a), n)
-                self.assertEqual(b.endian(), default_endian)
+                self.assertEqual(a.endian(), default_endian)
 
-            a = urandom(1000)
-            b = urandom(1000)
-            self.assertNotEqual(a, b)
-            self.assertTrue(400 < a.count() < 600)
-            self.assertTrue(400 < b.count() < 600)
+            for endian in 'big', 'little':
+                a = urandom(11, endian)
+                self.assertEqual(len(a), 11)
+                self.assertEqual(a.endian(), endian)
+
+    def test_count(self):
+        a = urandom(1000)
+        b = urandom(1000)
+        self.assertNotEqual(a, b)
+        self.assertTrue(400 < a.count() < 600)
+        self.assertTrue(400 < b.count() < 600)
 
     def test_wrong_args(self):
         self.assertRaises(TypeError, urandom)
@@ -111,7 +118,7 @@ class TestsRandom(unittest.TestCase):
         self.assertRaises(TypeError, urandom, 0, 1)
         self.assertRaises(ValueError, urandom, 0, 'foo')
 
-tests.append(TestsRandom)
+tests.append(TestsURandom)
 
 # ---------------------------------------------------------------------------
 
@@ -370,25 +377,33 @@ class TestsCount_N(unittest.TestCase, Util):
 
     @staticmethod
     def count_n(a, n):
-        "return the index i for which a[:i].count() == n"
+        "return lowest index i for which a[:i].count() == n"
         i, j = n, a.count(1, 0, n)
         while j < n:
-            if a[i]:
-                j += 1
+            j += a[i]
             i += 1
         return i
 
     def check_result(self, a, n, i):
         self.assertEqual(a.count(1, 0, i), n)
-        if i > 0:
+        if i == 0:
+            self.assertEqual(n, 0)
+        else:
             self.assertTrue(a[i - 1])
 
+    def test_empty(self):
+        a = bitarray()
+        self.assertEqual(count_n(a, 0), 0)
+        self.assertRaises(ValueError, count_n, a, 1)
+        self.assertRaises(TypeError, count_n, '', 0)
+        self.assertRaises(TypeError, count_n, a, 7.0)
+        self.assertRaises(TypeError, count_n, a, 0, 0)
+
     def test_simple(self):
         a = bitarray('111110111110111110111110011110111110111110111000')
         b = a.copy()
         self.assertEqual(len(a), 48)
         self.assertEqual(a.count(), 37)
-        self.assertRaises(TypeError, count_n, '', 0)
         self.assertEqual(count_n(a, 0), 0)
         self.assertEqual(count_n(a, 20), 23)
         self.assertEqual(count_n(a, 37), 45)
@@ -398,8 +413,7 @@ class TestsCount_N(unittest.TestCase, Util):
                                  count_n, a, 49) # n > len(a)
         self.assertRaisesMessage(ValueError, "n exceeds total count",
                                  count_n, a, 38) # n > a.count()
-        self.assertRaises(TypeError, count_n, a, "7")
-        for n in range(0, 37):
+        for n in range(0, 38):
             i = count_n(a, n)
             self.check_result(a, n, i)
             self.assertEqual(a[:i].count(), n)
@@ -410,39 +424,51 @@ class TestsCount_N(unittest.TestCase, Util):
         a = frozenbitarray('001111101111101111101111100111100')
         self.assertEqual(len(a), 33)
         self.assertEqual(a.count(), 24)
-        self.assertRaises(TypeError, count_n, '', 0)
         self.assertEqual(count_n(a, 0), 0)
         self.assertEqual(count_n(a, 10), 13)
         self.assertEqual(count_n(a, 24), 31)
         self.assertRaises(ValueError, count_n, a, -1) # n < 0
         self.assertRaises(ValueError, count_n, a, 25) # n > a.count()
         self.assertRaises(ValueError, count_n, a, 34) # n > len(a)
-        self.assertRaises(TypeError, count_n, a, "7")
+        for n in range(0, 25):
+            self.check_result(a, n, count_n(a, n))
+
+    def test_ones(self):
+        n = randint(1, 100000)
+        a = bitarray(n)
+        a.setall(1)
+        self.assertEqual(count_n(a, n), n)
+        self.assertRaises(ValueError, count_n, a, n + 1)
+        for _ in range(20):
+            i = randint(0, n)
+            self.assertEqual(count_n(a, i), i)
+
+    def test_one_set(self):
+        n = randint(1, 100000)
+        a = bitarray(n)
+        a.setall(0)
+        self.assertEqual(count_n(a, 0), 0)
+        self.assertRaises(ValueError, count_n, a, 1)
+        for _ in range(20):
+            a.setall(0)
+            i = randint(0, n - 1)
+            a[i] = 1
+            self.assertEqual(count_n(a, 1), i + 1)
+            self.assertRaises(ValueError, count_n, a, 2)
 
     def test_large(self):
-        for N in list(range(100)) + [1000, 10000, 100000]:
+        for N in list(range(50)) + [randint(50, 250000) for _ in range(10)]:
             a = bitarray(N)
             v = randint(0, 1)
             a.setall(not v)
             for _ in range(randint(0, min(N, 100))):
                 a[randint(0, N - 1)] = v
             tc = a.count()      # total count
-            n = randint(0, tc)
-            self.check_result(a, n, count_n(a, n))
             self.check_result(a, tc, count_n(a, tc))
             self.assertRaises(ValueError, count_n, a, tc + 1)
-
-    def test_one_set(self):
-        N = 100000
-        for _ in range(10):
-            a = bitarray(N)
-            a.setall(0)
-            self.assertEqual(count_n(a, 0), 0)
-            self.assertRaises(ValueError, count_n, a, 1)
-            i = randint(0, N - 1)
-            a[i] = 1
-            self.assertEqual(count_n(a, 1), i + 1)
-            self.assertRaises(ValueError, count_n, a, 2)
+            for _ in range(20):
+                n = randint(0, tc)
+                self.check_result(a, n, count_n(a, n))
 
     def test_random(self):
         for a in self.randombitarrays():
